
BFC_AB = {}; -- Initialize the BFC_AB namespace

local BFC_AB_REZ_INTERVAL = 32; -- rez wave every ~30s
local BFC_AB_NODE_CAP_TIME = 64;



local gyclosestdist = 999;
local playerfaction;
local nodetrans;
BFC_AB.timers = {};

local BFC_AB_GRAVEYARDS = {
	{	x = 0.395845,
		y = 0.261748,
		id = BFC_Strings.AB_Nodes.stables,
		name = BFC_Strings.AB.stables,
	},
	{	x = 0.613111,
		y = 0.253503,
		id = BFC_Strings.AB_Nodes.mine,
		name = BFC_Strings.AB.mine
	},
	{	x = 0.367385,
		y = 0.628251,
		id = BFC_Strings.AB_Nodes.mill,
		name = BFC_Strings.AB.mill
	},
	{	x = 0.514248,
		y = 0.419996,
		id = BFC_Strings.AB_Nodes.blacksmith,
		name = BFC_Strings.AB.blacksmith
	},
	{	x = 0.611782,
		y = 0.575198,
		id = BFC_Strings.AB_Nodes.farm,
		name = BFC_Strings.AB.farm
	},
	{	x = 0.334840,
		y = 0.131217,
		id = "trollbane",
		name = BFC_Strings.AB.trollbane
	},
	{	x = 0.694644,
		y = 0.678682,
		id = "defilers",
		name = BFC_Strings.AB.defilers
	},
};

local BFC_AB_FACTIONCOLORS = {
	Horde = "|cffff2020",
	Alliance = "|cff6060ff",
	Neutral = "|cff999999",
};

local graveyardfactions = {};

function BFC_AB.Onload()
	-- Create the helper module and register it with the BFC system.
	local modobj = {
		Name = BFC_Strings.AB.modname, -- A unique name, doesn't need to be the zone name
		Author = "Vallerius", -- Give yourself credit. Not used yet, might appear later in a management screen
		Version = "1.0", -- Also unused for now
		Type = "ZONE_HELPER", -- Must be ZONE_HELPER for per-zone helper objects
		SetActive = BFC_AB.SetActive, -- Function to enable/disable the helper
		IsActiveZone = BFC_AB.IsActiveZone, -- Function to determine if we're in a target zone
		OptionsFrame = "BFC_AB_OptionsFrame", -- Frame to display helper options in
	};
	BFC_Common.RegisterHelperModule(modobj);
	nodetrans = BFC_Util.FlipKeysAndValues(BFC_Strings.AB_Nodes);
	playerfaction = UnitFactionGroup("player");
end


-- Called by the module dispatcher whenever we enter/leave an active zone
function BFC_AB.SetActive(state)
	if(state == true) then
		UIDropDownMenu_Initialize(BFC_AB_OptionsDropDown, BFC_AB.OptionsDropDown_Initialize, "MENU");
		
		BFC_AB_Frame:RegisterEvent("AREA_SPIRIT_HEALER_IN_RANGE");
		BFC_AB_Frame:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE"); 
		BFC_AB_Frame:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE"); 
		BFC_AB_Frame:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
		
		-- Reset the mod state so that it's fresh every time
		
		BFC_AB.RezTimer = BFC_AB_REZ_INTERVAL
		BFC_AB.RezTimerActive = false;
		
		graveyardfactions[BFC_Strings.AB_Nodes.blacksmith] = "Neutral";
		graveyardfactions[BFC_Strings.AB_Nodes.farm] = "Neutral";
		graveyardfactions[BFC_Strings.AB_Nodes.mine] = "Neutral";
		graveyardfactions[BFC_Strings.AB_Nodes.mill] = "Neutral";
		graveyardfactions[BFC_Strings.AB_Nodes.stables] = "Neutral";
		graveyardfactions["trollbane"] = "Alliance";
		graveyardfactions["defilers"] = "Horde";
		
		-- The update function will be called every frame, even if the AB helper frame isn't visible.
		BFC_Common.RegisterUpdateFunction("BFC_AB", BFC_AB.OnUpdate);
		
		-- Set up the timers
		BFC_AB.timers = {};
		BFC_AB.timers.blacksmith = BFC_Timer:new();
		BFC_AB.timers.blacksmith:SetFontString(BFC_AB_Blacksmith);
		BFC_AB.timers.blacksmith:SetText(BFC_Strings.AB.blacksmith);
		BFC_AB.timers.blacksmith:SetTimerColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.blacksmith:SetLabelColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.blacksmith:SetCallback(BFC_AB.NodeTimerCallback);
		--BFC_AB.tSmith:SetTime(BFC_AB_NODE_CAP_TIME);
		--BFC_AB.tSmith:Start();
		
		BFC_AB.timers.farm = BFC_Timer:new();
		BFC_AB.timers.farm:SetFontString(BFC_AB_Farm);
		BFC_AB.timers.farm:SetText(BFC_Strings.AB.farm);
		BFC_AB.timers.farm:SetTimerColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.farm:SetLabelColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.farm:SetCallback(BFC_AB.NodeTimerCallback);
		--BFC_AB.tFarm:SetTime(BFC_AB_NODE_CAP_TIME);
		--BFC_AB.tFarm:Start();
		
		BFC_AB.timers.mill = BFC_Timer:new();
		BFC_AB.timers.mill:SetFontString(BFC_AB_Mill);
		BFC_AB.timers.mill:SetText(BFC_Strings.AB.mill);
		BFC_AB.timers.mill:SetTimerColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.mill:SetLabelColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.mill:SetCallback(BFC_AB.NodeTimerCallback);
		--BFC_AB.tMill:SetTime(BFC_AB_NODE_CAP_TIME);
		--BFC_AB.tMill:Start();
		
		BFC_AB.timers.mine = BFC_Timer:new();
		BFC_AB.timers.mine:SetFontString(BFC_AB_Mine);
		BFC_AB.timers.mine:SetText(BFC_Strings.AB.mine);
		BFC_AB.timers.mine:SetTimerColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.mine:SetLabelColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.mine:SetCallback(BFC_AB.NodeTimerCallback);
		--BFC_AB.tMine:SetTime(BFC_AB_NODE_CAP_TIME);
		--BFC_AB.tMine:Start();
		
		BFC_AB.timers.stables = BFC_Timer:new();
		BFC_AB.timers.stables:SetFontString(BFC_AB_Stables);
		BFC_AB.timers.stables:SetText(BFC_Strings.AB.stables);
		BFC_AB.timers.stables:SetTimerColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.stables:SetLabelColorCode(BFC_AB_FACTIONCOLORS.neutral);
		BFC_AB.timers.stables:SetCallback(BFC_AB.NodeTimerCallback);
		--BFC_AB.tStables:SetTime(BFC_AB_NODE_CAP_TIME);
		--BFC_AB.tStables:Start();
		
		
		-- Zone-specific comms menu
		BFC_Radio.SetLocalMenuItems(BFC_Strings.CommsMenu_AB);
		
		-- Receive rez time notifications
		BFC_Comms.RegisterMessage("AB", "REZ", BFC_AB.HandleRezMessage);
		BFC_AB_Frame:Show();
	else
		BFC_AB_Frame:UnregisterEvent("AREA_SPIRIT_HEALER_IN_RANGE");
		BFC_AB_Frame:UnregisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE"); 
		BFC_AB_Frame:UnregisterEvent("CHAT_MSG_BG_SYSTEM_HORDE"); 
		BFC_AB_Frame:UnregisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
		
		BFC_Comms.UnregisterMessage("AB", "REZ");
		BFC_Radio.SetLocalMenuItems(nil);
		
		-- No need for the update when we're not active.
		BFC_Common.UnregisterUpdateFunction("BFC_AB");
		
		-- Stop and clean up any timers
		for i,v in ipairs(BFC_AB.timers) do
			v:Stop();
			v = nil;
		end
		
		BFC_AB_Frame:Hide();
	end
end

-- Called by the module dispatcher to determine if we're in an active zone
function BFC_AB.IsActiveZone(zone)
	if(zone == BFC_Strings.AB.zone) then
		return true;
	else
		return false;
	end
end


-- Called by the common OnUpdate handler (registered/unregistered in SetActive)
function BFC_AB.OnUpdate(elapsed)
	-- FIXME: throttle this
	local gy = BFC_AB.LocateNearestGraveyard();
	if(gy ~= nil) then
		BFC_AB_RezLoc:SetText(string.format(BFC_Strings.AB.rezloc, gy.name));
	end
end


-- Called whenever one of the node timers counts to zero
function BFC_AB.NodeTimerCallback(timer)
	timer:SetTime(BFC_AB_REZ_INTERVAL);
	timer:Start();
end


-- Locate the nearest captured graveyard, that is where the player will rez at
function BFC_AB.LocateNearestGraveyard()
	local px,py = GetPlayerMapPosition("player");
	local distsq, a, b;
	local closest = nil;
	local closestDist = 9999; -- this is plenty, dists should be < 1 anyway
	
	for i,v in ipairs(BFC_AB_GRAVEYARDS) do
		a = math.abs(px - v.x);
		b = math.abs(py - v.y);
		distsq = a*a + b*b;
		if(distsq < closestDist and graveyardfactions[v.id] and graveyardfactions[v.id] == playerfaction) then
			closestDist = distsq;
			closest = v;
			--BFC.Log(BFC.LOG_ERROR, "ID is " .. v.id);
			--if(graveyardfactions[v.id]) then
			--	BFC.Log(BFC.LOG_ERROR, "Faction is " .. graveyardfactions[v.id]);
			--	BFC.Log(BFC.LOG_ERROR, "PF is " .. playerfaction);
			--end
		end
	end
	
	return closest;
end


function BFC_AB.HandleRezMessage(player, timeleft)
	BFC.Log(BFC.LOG_DEBUG, "Syncing rez timer to " .. timeleft[1]);
	BFC_AB.SyncTimer(timeleft[1]);
end


-- Set the rez timer to a new value
function BFC_AB.SyncTimer(secondsLeft)
	BFC_AB.CurTime = GetTime();
	BFC_AB.RezTimer = secondsLeft;
	BFC_AB.RezTimerActive = true;
end


-- Your standard event handler
function BFC_AB.OnEvent()
	if(event == "AREA_SPIRIT_HEALER_IN_RANGE" and UnitIsGhost("player")) then
		local timeToRez = GetAreaSpiritHealerTime();
		BFC_AB.SyncTimer(timeToRez);
		BFC_Comms.SendMessage("AB", "REZ", timeToRez);
	elseif(event == "UPDATE_BATTLEFIELD_SCORE") then
		BFC_AB.UpdateClassInfo();
	elseif(event == "CHAT_MSG_BG_SYSTEM_ALLIANCE") then
		BFC_AB.ParseNodeEvent(arg1, "Alliance");
	elseif(event == "CHAT_MSG_BG_SYSTEM_HORDE") then
		BFC_AB.ParseNodeEvent(arg1, "Horde");
	end
end


-- Figure out what happened to the node
function BFC_AB.ParseNodeEvent(text, faction)
	local i, _, node = string.find(text, BFC_Strings.AB.event_assaulted);
	local timer;
	if(i) then
		--BFC.Log(BFC.LOG_ERROR, "Node is " .. node);
		graveyardfactions[node] = "Neutral";
		timer = BFC_AB.timers[nodetrans[node]];
		timer:SetTime(BFC_AB_NODE_CAP_TIME);
		timer:SetCallback(nil);
		timer:SetTimerColorCode(BFC_AB_FACTIONCOLORS[faction]);
		timer:SetLabelColorCode(BFC_AB_FACTIONCOLORS.neutral);
		timer:Start();
	end
	
	local i, _, node = string.find(text, BFC_Strings.AB.event_taken);
	if(i) then
		--BFC.Log(BFC.LOG_ERROR, "Node is " .. node);
		graveyardfactions[node] = faction;
		timer = BFC_AB.timers[nodetrans[node]];
		--timer:SetCallback(BFC_AB.NodeTimerCallback);
		timer:SetTimerColorCode(BFC_AB_FACTIONCOLORS[faction]);
		timer:SetLabelColorCode(BFC_AB_FACTIONCOLORS[faction]);
		timer:Stop();
		timer:SetTime(-1);
		--timer:Start();
	end
	
	local i, _, node = string.find(text, BFC_Strings.AB.event_claims);
	if(i) then
		--BFC.Log(BFC.LOG_ERROR, "Node is " .. node);
		graveyardfactions[node] = "Neutral";
		--BFC.Log(BFC.LOG_ERROR, "Node is " .. node);
		timer = BFC_AB.timers[nodetrans[node]];
		timer:SetTime(BFC_AB_NODE_CAP_TIME);
		timer:SetCallback(nil);
		timer:SetTimerColorCode(BFC_AB_FACTIONCOLORS[faction]);
		timer:Start();
	end
		
end

function BFC_AB.TargetCarrier(carrier)
	if(carrier and carrier ~= "" and carrier ~= BFC_Strings.AB["atbase"] and carrier ~= BFC_Strings.AB["dropped"]) then
		TargetByName(carrier);
	end
end



-- Called when the frame is right-clicked
function BFC_AB.ShowSettingsFrame()
	ToggleDropDownMenu(1, nil, BFC_AB_OptionsDropDown, this:GetName(), 0, 0);
end



function BFC_AB.OptionsDropDown_Initialize()
	local checked;
	local info = {};
	
	info.text = BFC_Strings.AB.zone;
	info.justifyH = "CENTER";
	info.isTitle = 1;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	
	-- Show battlefield score
	if ( BFC_Options.get("ab_showScore") ) then
		checked = 1;
	end
	info.text = BFC_Strings.AB_Options.showscore;
	info.func = BFC_AB.ToggleScore;
	info.checked = checked;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	
	-- Hide the default scoreboard
	if ( BFC_Options.get("ab_hideScoreBoard") ) then
		checked = 1;
	end
	info.text = BFC_Strings.AB_Options.hidescoreboard;
	info.func = BFC_AB.ToggleScoreBoard;
	info.checked = checked;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	
	-- AB window lock
	checked = nil;
	if ( BFC_Options.get("ab_locked") ) then
		checked = 1;
	end
	info.text = BFC_Strings.AB_Options.lockwindow;
	info.func = BFC_AB.ToggleLock;
	info.checked = checked;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

	-- Show timers
	checked = nil;
	if ( BFC_Options.get("ab_timers") ) then
		checked = 1;
	end
	info = {};
	info.text = BFC_Strings.AB_Options.showtimers;
	info.func = BFC_AB.ToggleTimers;
	info.checked = checked;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	
	-- Hide
	--checked = nil;
	--info = {};
	--info.text = BFC_Strings.AB_Options.hidewindow;
	--info.notCheckable = 1;
	--info.func = BFC_AB.HideInfoWindow;
	--UIDropDownMenu_AddButton(info);
end


function BFC_AB.ToggleScore()

end


function BFC_AB.ToggleScoreBoard()

end


function BFC_AB.ToggleLock()

end

function BFC_AB.ToggleTimers()

end